//
//
// hello.button.45.c
//
// button hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld
// 10/31/10
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <string.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay
#define awk_delay() _delay_ms(100) // LED flash delay

#define input_port PORTB
#define input_direction DDRB
#define input_pin_in (1 << PB4)
#define input_pin_out (1 << PB3)
#define input_pins PINB
#define serial_port PORTB
#define serial_pins PINB
#define serial_direction DDRB
#define serial_pin_in (1 << PB1)
#define serial_pin_out (1 << PB2)

#define awk_port PORTB
#define awk_direction DDRB
#define awk_pin (1 << PB0)

#define node_id_in '3' // in node
#define node_id_out '4' // out node

void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
	//
	// read character into rxbyte on pins pin
	//    assumes line driver (inverts bits)
	//
	*rxbyte = 0;
	while (pin_test(*pins,pin))
	//
	// wait for start bit
	//
	;
	//
	// delay to middle of first data bit
	//
	half_bit_delay();
	bit_delay();
	//
	// unrolled loop to read data bits
	//
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 0);
	else
	*rxbyte |= (0 << 0);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 1);
	else
	*rxbyte |= (0 << 1);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 2);
	else
	*rxbyte |= (0 << 2);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 3);
	else
	*rxbyte |= (0 << 3);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 4);
	else
	*rxbyte |= (0 << 4);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 5);
	else
	*rxbyte |= (0 << 5);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 6);
	else
	*rxbyte |= (0 << 6);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 7);
	else
	*rxbyte |= (0 << 7);
	//
	// wait for stop bit
	//
	bit_delay();
	half_bit_delay();
}

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

void flash() {
	//
	// LED flash delay
	//
	clear(awk_port, awk_pin);
	awk_delay();
	set(awk_port, awk_pin);
}

int main(void) {
   //
   // main
   //
   static char chr;
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize pins
   //
   set(serial_port, serial_pin_out);
   input(serial_direction, serial_pin_out);
   set(awk_port, awk_pin);
   output(awk_direction, awk_pin);
   set(input_port, input_pin_in); // turn on pull-up
   input(input_direction, input_pin_in);
   set(input_port, input_pin_out); // turn on pull-up
   input(input_direction, input_pin_out);
   //
   // main loop
   //
   while (1) {
      get_char(&serial_pins, serial_pin_in, &chr);
      if (chr == node_id_in) {
         flash(); // awk query
	     output(serial_direction, serial_pin_out);
	     //
	     // send framing
	     //
	     put_char(&serial_port, serial_pin_out, 1);
	     char_delay();
	     put_char(&serial_port, serial_pin_out, 2);
	     char_delay();
	     put_char(&serial_port, serial_pin_out, 3);
	     char_delay();
	     put_char(&serial_port, serial_pin_out, 4);
	     char_delay();
	     //
         // in button down
         //
         if (0 == pin_test(input_pins,input_pin_in)){
            put_char(&serial_port, serial_pin_out, 'd');						
	        clear(awk_port, awk_pin); // leave awk led on
            }
         //
         // in button up
         //
         else {
            put_char(&serial_port, serial_pin_out, 'u');				
	        set(awk_port, awk_pin); // turn awk led off	
	     }
      }
      if (chr == node_id_out) {
	      flash(); // awk query
	      output(serial_direction, serial_pin_out);
	      //
	      // send framing
	      //
	      put_char(&serial_port, serial_pin_out, 1);
	      char_delay();
	      put_char(&serial_port, serial_pin_out, 2);
	      char_delay();
	      put_char(&serial_port, serial_pin_out, 3);
	      char_delay();
	      put_char(&serial_port, serial_pin_out, 4);
	      char_delay();
	      //
	      // in button down
	      //
	      if (0 == pin_test(input_pins,input_pin_out)){
		      put_char(&serial_port, serial_pin_out, 'd');
		      clear(awk_port, awk_pin); // leave awk led on
	      }
	      //
	      // in button up
	      //
	      else {
		      put_char(&serial_port, serial_pin_out, 'u');
		      set(awk_port, awk_pin); // turn awk led off
	      }
      }
   }
}
