//
//
// hello.H-bridge.44.DC.c
//
// H-bridge DC motor hello-world
//
// Neil Gershenfeld
// 11/18/12
//
// (c) Massachusetts Institute of Technology 2012
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay
#define awk_delay() _delay_ms(100) // LED flash delay
#define error_delay() _delay_ms(50) // error flash delay

#define serial_port PORTB
#define serial_pins PINB
#define serial_direction DDRB
#define serial_pin_in (1 << PB0)
#define serial_pin_out (1 << PB1)

#define awk_port PORTA
#define awk_direction DDRA
#define awk_pin (1 << PA6)

#define sol_port PORTA
#define sol_direction DDRA
#define sol_pin (1 << PA0)

#define max_buffer 5

void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
	//
	// read character into rxbyte on pins pin
	//    assumes line driver (inverts bits)
	//
	*rxbyte = 0;
	while (pin_test(*pins,pin))
	//
	// wait for start bit
	//
	;
	//
	// delay to middle of first data bit
	//
	half_bit_delay();
	bit_delay();
	//
	// unrolled loop to read data bits
	//
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 0);
	else
	*rxbyte |= (0 << 0);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 1);
	else
	*rxbyte |= (0 << 1);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 2);
	else
	*rxbyte |= (0 << 2);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 3);
	else
	*rxbyte |= (0 << 3);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 4);
	else
	*rxbyte |= (0 << 4);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 5);
	else
	*rxbyte |= (0 << 5);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 6);
	else
	*rxbyte |= (0 << 6);
	bit_delay();
	if pin_test(*pins,pin)
	*rxbyte |= (1 << 7);
	else
	*rxbyte |= (0 << 7);
	//
	// wait for stop bit
	//
	bit_delay();
	half_bit_delay();
}
void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
	//
	// send character in txchar on port pin
	//    assumes line driver (inverts bits)
	//
	// start bit
	//
	clear(*port,pin);
	bit_delay();
	//
	// unrolled loop to write data bits
	//
	if bit_test(txchar,0)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,1)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,2)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,3)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,4)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,5)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,6)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	if bit_test(txchar,7)
	set(*port,pin);
	else
	clear(*port,pin);
	bit_delay();
	//
	// stop bit
	//
	set(*port,pin);
	bit_delay();
	//
	// char delay
	//
	bit_delay();
}

void flash() {
	//
	// LED flash delay
	//
	clear(awk_port, awk_pin);
	awk_delay();
	set(awk_port, awk_pin);
}

void error_flash() {
	//
	// LED flash delay
	//
	clear(awk_port, awk_pin);
	error_delay();
	set(awk_port, awk_pin);
	clear(awk_port, awk_pin);
	error_delay();
	set(awk_port, awk_pin);
}
int get_sensor() {
   static char chr;
   static char buffer[max_buffer] = {0};
   static int index;
   index = 0;
   while (1) {
      //
      // find framing
      //
      get_char(&serial_pins, serial_pin_in, &chr);
      buffer[index++] = chr;
      if (index == (max_buffer-1 || buffer[0] != '1')) {  
		  break;   
	  }
	  //if ((index == 0 && chr=='1')||(...)
   }
   if (buffer[0] == '1' && buffer[1] == '2' && buffer[2] == '3' && buffer[3] == '4'){
      return buffer[4];	
   } 
   else {
      return 0;
   }
}

int main(void) {
   //
   // main
   //
   static char node1;
   static char node2;
   static char node3;
   static char node4;
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize pins
   //
   set(serial_port, serial_pin_out);
   input(serial_direction, serial_pin_out);
   set(awk_port, awk_pin);
   output(awk_direction, awk_pin);
   //
   // initialize solenoid pins
   //
   clear(sol_port, sol_pin);
   output(sol_direction, sol_pin);
   //
   // main loop
   //
   while (1) {
	  //
      // Get input from prox 1
      //
      output(serial_direction, serial_pin_out);
      put_char(&serial_port, serial_pin_out, '1');
      flash();
	  node1 = get_sensor();
	  // 
	  // troubleshooting
	  //
	  //if (node1 == 0){
	//	  error_flash();
	  //}
	  //
	  // Get input from prox 2
	  //
	  output(serial_direction, serial_pin_out);
      flash();
	  put_char(&serial_port, serial_pin_out, '2');
	  flash();
	  node2 = get_sensor();
	  //
	  // troubleshooting
	  //
	  //if (node2 == 0){
	  //	  error_flash();
	  //}	  
	  //
	  // Get input from button 3
	  //
	  output(serial_direction, serial_pin_out);
	  flash();
	  flash();
	  put_char(&serial_port, serial_pin_out, '3');
	  flash();
	  node3 = get_sensor();
	  //
	  // troubleshooting
	  //
	  //if (node3 == 0){
	//	  error_flash();
	  //}  
	  //
	  // Get input from button 4
	  //
	  output(serial_direction, serial_pin_out);
	  flash();
	  flash();
	  flash();
	  put_char(&serial_port, serial_pin_out, '4');
	  flash();
	  node4 = get_sensor();
	  //
	  // troubleshooting
	  //
	  //if (node4 == 0){
	//	  error_flash();
	  //} 
      //
	  // Set solenoid if extending
	  //
	  //if (node1 == 'h' && node2 == 'l' && node4 != 'd'){
	  //	  set(sol_port, sol_pin);
	  //}
	  //
	  // Set solenoid if pushing in
	  //
	  //if (node1 == 'l' && node3 != 'd'){
	  //	  set(sol_port, sol_pin);
	  //}
	  //
	  // Testing
	  //
	  if (node1 == 'h' && node4 == 'd'){
	  	  set(sol_port, sol_pin);
	  }	  
	  else {
		  clear(sol_port, sol_pin);
	  }
   }
}
